#include "MciDisc.h"
#include "discid.h"

CMciDisc::CMciDisc(const char *device)
  : m_track_count(0)
{
  char strRet[MCI_MAXLEN];
  char strCommand[MCI_MAXLEN];

  sprintf(strCommand, "open %s wait", device);
  MCIERROR status = SendString(strCommand, strRet);
  if (status == S_OK)
  {
    status = ReadTrackCount(device);
    if (status == S_OK)
    {
      status = CreateTracks(device);
      if (status != S_OK)
        m_track_count = 0; // fail
    }
    else
      fprintf(stderr, "Can not read track count");
  }
  else
    fprintf(stderr, "Can not open device %s\n", device);
}

CMciDisc::~CMciDisc()
{
  for (unsigned track = 0; track < m_track_count; ++track)
    delete m_track[track];
}

unsigned CMciDisc::GetTrackCount() const
{
  return m_track_count;
}

unsigned long CMciDisc::GetStartFrame(unsigned track) const
{
  return m_track[track]->start_frame;
}

unsigned long CMciDisc::GetFrames(unsigned track) const
{
  return m_track[track]->frames;
}

MCIERROR CMciDisc::ReadTrackCount(const char *device)
{
  char strRet[MCI_MAXLEN];
  char strCommand[MCI_MAXLEN];

  sprintf(strCommand, "status %s number of tracks", device);
  MCIERROR status = SendString(strCommand, strRet);
  if (status == S_OK)
  {
    if (sscanf(strRet, "%d", &m_track_count) != 1)
    {
      fprintf(stderr, "Can not read track count\n");
      status = static_cast<MCIERROR>(-1);
    }
  }

  return status;
}

MCIERROR CMciDisc::CreateTracks(const char *device)
{
  MCIERROR status = S_OK;

  char strRet[MCI_MAXLEN];
  char strCommand[MCI_MAXLEN];
  for (unsigned track = 0; (status == S_OK) && (track < m_track_count); ++track)
  {
    sprintf(strCommand, "set %s time format msf", device);
    MCIERROR status = SendString(strCommand, strRet);
    if (status == S_OK)
    {
      sprintf(strCommand, "status %s position track %d", device, track + 1);
      status = SendString(strCommand, strRet);
      if (status == S_OK)
      {
        unsigned start_min;
        unsigned start_sec;
        unsigned start_frame;
        if (sscanf(strRet, "%02d:%02d:%02d", &start_min, &start_sec, &start_frame) == 3)
        {
          sprintf(strCommand, "status %s length track %d", device, track + 1);
          status = SendString(strCommand, strRet);
          if (status == S_OK)
          {
            unsigned len_min;
            unsigned len_sec;
            unsigned len_frame;
            if (sscanf(strRet, "%02d:%02d:%02d", &len_min, &len_sec, &len_frame) == 3)
            {
              Track *pTrack = new Track;

              pTrack->start_frame =
                60 * FRAMES_PER_SECOND * start_min + FRAMES_PER_SECOND * start_sec + start_frame;

              // Windows MCI-Error: returns len 1 short
              pTrack->frames =
                60 * FRAMES_PER_SECOND * len_min + FRAMES_PER_SECOND * len_sec + len_frame + 1;

              m_track.push_back(pTrack);
            }
            else
            {
              status = static_cast<MCIERROR>(-1);
              fprintf(stderr, "Can not read length of track\n");
            }
          }
        }
        else
        {
          status = static_cast<MCIERROR>(-1);
          fprintf(stderr, "Can not read position of track\n");
        }
      }
    }
  }

  return status;
}

MCIERROR CMciDisc::SendString(const char *command, char *result, unsigned buflen)
{
  MCIERROR status = mciSendString(command, result, buflen, NULL);
  if (status != S_OK)
  {
    char strError[MCI_MAXLEN];
    mciGetErrorString(status, strError, MCI_MAXLEN);

    char strMessage[MCI_MAXLEN];
    sprintf(strMessage, "SendString command: '%s' returned:\n  '%s'", command, strError);

    fprintf(stderr, "%s\n", strMessage);
  }

  return status;
}
